@@ -5,13 +5,14 @@
 #include <QProcess>
 #include <QMessageBox>
 #include <QFontMetrics>
+#include <QtNetwork/QNetworkSession>
 
-#include "settings.h"
 #include "trayicon.h"
-#include "unreadcounter.h"
-#include "dialogsettings.h"
+#include "unreadmonitor.h"
 #include "windowtools.h"
 #include "utils.h"
+#include "autoupdater.h"
+#include "birdtrayapp.h"
 
 TrayIcon::TrayIcon(bool showSettings)
 {
@@ -21,23 +22,32 @@ TrayIcon::TrayIcon(bool showSettings)
 
     mIgnoredUnreadEmails = 0;
     mUnreadCounter = 0;
-    mUnreadMonitor = 0;
+
+    // Context menu
+    mSystrayMenu = new QMenu();
+    setContextMenu( mSystrayMenu );
 
     mMenuShowHideThunderbird = 0;
     mMenuIgnoreUnreads = 0;
     mThunderbirdProcess = 0;
-
+#ifdef Q_OS_WIN
+    mThunderbirdUpdaterProcess = ProcessHandle::create(Utils::getThunderbirdUpdaterName());
+    connect( mThunderbirdUpdaterProcess, &ProcessHandle::finished,
+            this, &TrayIcon::tbUpdaterProcessFinished );
+#endif /* Q_OS_WIN */
+    
     mThunderbirdWindowExists = false;
     mThunderbirdWindowExisted = false;
     mThunderbirdWindowHide = false;
     connect(QApplication::instance(), &QApplication::aboutToQuit, this, &TrayIcon::onQuit);
 
-    mThunderbirdStartTime = QDateTime::currentDateTime().addSecs( pSettings->mLaunchThunderbirdDelay );
+    Settings* settings = BirdtrayApp::get()->getSettings();
+    mThunderbirdStartTime = QDateTime::currentDateTime().addSecs(settings->mLaunchThunderbirdDelay);
 
     mWinTools = WindowTools::create();
 
     // If the settings are not yet configure, pop up the message
-    if ( showSettings || (pSettings->mFolderNotificationColors.isEmpty() && QMessageBox::question(
+    if ( showSettings || (settings->mFolderNotificationColors.isEmpty() && QMessageBox::question(
             nullptr, tr( "Would you like to set up Birdtray?" ),
             tr( "You have not yet configured any email folders to monitor. "
                 "Would you like to do it now?") ) == QMessageBox::Yes )) {
@@ -59,6 +69,31 @@ TrayIcon::TrayIcon(bool showSettings)
     updateState();
     updateIcon();
     show();
+    
+    if (settings->mUpdateOnStartup) {
+        doAutoUpdateCheck();
+    }
+}
+
+TrayIcon::~TrayIcon() {
+    if (settingsDialog != nullptr) {
+        settingsDialog->deleteLater();
+    }
+#ifdef Q_OS_WIN
+    mThunderbirdUpdaterProcess->deleteLater();
+#endif /* Q_OS_WIN */
+    if (networkConnectivityManager != nullptr) {
+        networkConnectivityManager->deleteLater();
+        networkConnectivityManager = nullptr;
+    }
+    if (mUnreadMonitor != nullptr) {
+        mUnreadMonitor->deleteLater();
+        mUnreadMonitor = nullptr;
+    }
+}
+
+WindowTools* TrayIcon::getWindowTools() const {
+    return mWinTools;
 }
 
 void TrayIcon::unreadCounterUpdate( unsigned int total, QColor color )
@@ -75,8 +110,11 @@ void TrayIcon::unreadCounterError(QString message)
     qWarning("UnreadCounter generated an error: %s", qPrintable(message) );
 
     mCurrentStatus = message;
+    if (mUnreadMonitor == nullptr) {
+        return;
+    }
     mUnreadMonitor->deleteLater();
-    mUnreadMonitor = 0;
+    mUnreadMonitor = nullptr;
 
     mUnreadCounter = 0;
     updateIcon();
@@ -93,7 +131,8 @@ static unsigned int largestFontSize(const QFont &font, int minfontsize, int maxf
     {
         cursize = minfontsize + (maxfontsize - minfontsize) / 2;
         testfont.setPointSize( cursize );
-        testfont.setWeight( pSettings->mNotificationFontWeight );
+        testfont.setWeight(
+                static_cast<int>(BirdtrayApp::get()->getSettings()->mNotificationFontWeight));
         QSize size = QFontMetrics( testfont ).size( Qt::TextSingleLine, text );
 
         if ( size.width() < rectsize.width() && size.height() <= rectsize.height() )
@@ -107,6 +146,7 @@ static unsigned int largestFontSize(const QFont &font, int minfontsize, int maxf
 
 void TrayIcon::updateIcon()
 {
+    Settings* settings = BirdtrayApp::get()->getSettings();
     // How many unread messages are here?
     unsigned int unread = mUnreadCounter;
 
@@ -129,14 +169,15 @@ void TrayIcon::updateIcon()
         unread -= mIgnoredUnreadEmails;
 
         // Are we blinking, and if not, should we be?
-        if ( unread > 0 && pSettings->mBlinkSpeed > 0 && mBlinkingTimeout == 0 )
-            enableBlinking( true );
+        if (unread > 0 && settings->mBlinkSpeed > 0 && mBlinkingTimeout == 0) {
+            enableBlinking(true);
+        }
 
         if ( unread == 0 && mBlinkingTimeout != 0 )
             enableBlinking( false );
     }
 
-    QPixmap temp( pSettings->getNotificationIcon().size() );
+    QPixmap temp(settings->getNotificationIcon().size());
     QPainter p;
 
     temp.fill( Qt::transparent );
@@ -152,16 +193,17 @@ void TrayIcon::updateIcon()
     else
         p.setOpacity( mBlinkingIconOpacity );
 
-    if ( unread != 0 && !pSettings->mNotificationIconUnread.isNull() )
-        p.drawPixmap( pSettings->mNotificationIconUnread.rect(), pSettings->mNotificationIconUnread );
-    else
-        p.drawPixmap( pSettings->getNotificationIcon().rect(), pSettings->getNotificationIcon() );
+    if (unread != 0 && !settings->mNotificationIconUnread.isNull()) {
+        p.drawPixmap(settings->mNotificationIconUnread.rect(), settings->mNotificationIconUnread);
+    } else {
+        p.drawPixmap(settings->getNotificationIcon().rect(), settings->getNotificationIcon());
+    }
 
-    p.setFont( pSettings->mNotificationFont );
+    p.setFont(settings->mNotificationFont);
 
     // Do we need to draw error sign?
-    if ( mUnreadMonitor == 0 || (pSettings->mMonitorThunderbirdWindow && !mThunderbirdWindowExists ) )
-    {
+    if (mUnreadMonitor == nullptr ||
+        (settings->mMonitorThunderbirdWindow && !mThunderbirdWindowExists)) {
         p.setOpacity( 1.0 );
         QPen pen( Qt::red );
         pen.setWidth( (temp.width() * 10) / 100 );
@@ -172,23 +214,35 @@ void TrayIcon::updateIcon()
     }
 
     // Do we need to draw the unread counter?
-    if ( unread > 0 && pSettings->mShowUnreadEmailCount )
-    {
+    if (unread > 0 && settings->mShowUnreadEmailCount) {
         // Find the suitable font size, starting from 4
         QString countvalue = QString::number( unread );
 
-        int fontsize = largestFontSize( pSettings->mNotificationFont,
-                                        pSettings->mNotificationMinimumFontSize,
-                                        pSettings->mNotificationMaximumFontSize,
-                                        countvalue,
-                                        temp.size() - QSize( 2, 2 ) );
+        int fontsize = static_cast<int>(largestFontSize(
+                settings->mNotificationFont,
+                static_cast<int>(settings->mNotificationMinimumFontSize),
+                static_cast<int>(settings->mNotificationMaximumFontSize),
+                countvalue, temp.size() - QSize(2, 2)));
 
-        pSettings->mNotificationFont.setPointSize( fontsize );
-        pSettings->mNotificationFont.setWeight( pSettings->mNotificationFontWeight );
-        QFontMetrics fm( pSettings->mNotificationFont );
+        settings->mNotificationFont.setPointSize(fontsize);
+        settings->mNotificationFont.setWeight(static_cast<int>(settings->mNotificationFontWeight));
+        QFontMetrics fm(settings->mNotificationFont);
         p.setOpacity( mBlinkingTimeout ? 1.0 - mBlinkingIconOpacity : 1.0 );
-        p.setPen( mUnreadColor );
-        p.drawText( (temp.width() - fm.width( countvalue )) / 2, (temp.height() - fm.height()) / 2 + fm.ascent(), countvalue );
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 11, 0))
+        int width = fm.horizontalAdvance(countvalue);
+#else
+        int width = fm.width(countvalue);
+#endif
+        QPainterPath textPath;
+        textPath.addText((temp.width() - width) / 2.0,
+                (temp.height() - fm.height()) / 2.0 + fm.ascent(),
+                settings->mNotificationFont, countvalue);
+        if (settings->mNotificationBorderWidth > 0
+            && settings->mNotificationBorderColor.isValid()) {
+            p.strokePath(textPath, QPen(
+                    settings->mNotificationBorderColor, settings->mNotificationBorderWidth));
+        }
+        p.fillPath(textPath, mUnreadColor);
     }
 
     p.end();
@@ -205,19 +259,17 @@ void TrayIcon::enableBlinking(bool enabled)
 {
     if ( enabled )
     {
+        Settings* settings = BirdtrayApp::get()->getSettings();
         mBlinkingIconOpacity = 1.0;
 
         // If we are using the alpha transition, we have to update icon more often
-        if ( pSettings->mBlinkingUseAlphaTransition )
-        {
-            mBlinkingDelta = pSettings->mBlinkSpeed / 100.0;
+        if (settings->mBlinkingUseAlphaTransition) {
+            mBlinkingDelta = settings->mBlinkSpeed / 100.0;
             mBlinkingTimeout = 100;
-        }
-        else
-        {
+        } else {
             // The blinking speed slider is a value from 0 to 30, so we make it 50x
             mBlinkingDelta = 0;
-            mBlinkingTimeout = pSettings->mBlinkSpeed * 50;
+            mBlinkingTimeout = settings->mBlinkSpeed * 50;
         }
 
         mBlinkingTimer.setInterval( mBlinkingTimeout );
@@ -249,12 +301,10 @@ void TrayIcon::updateState()
         {
             // If the window is found, we remember it
             if ( !mThunderbirdWindowExisted )
-            {
                 mThunderbirdWindowExisted = true;
 
-                if ( !mMenuShowHideThunderbird->isEnabled() )
-                    mMenuShowHideThunderbird->setEnabled( true );
-            }
+            if ( !mMenuShowHideThunderbird->isEnabled() )
+                mMenuShowHideThunderbird->setEnabled( true );
 
             // Hide the window if requested
             if ( mThunderbirdWindowHide )
@@ -265,29 +315,28 @@ void TrayIcon::updateState()
         }
         else
         {
+            Settings* settings = BirdtrayApp::get()->getSettings();
             // Thunderbird is not running. Has it run before?
-            if ( !mThunderbirdWindowExisted )
-            {
+            if (!mThunderbirdWindowExisted) {
                 // No. Shall we start it?
-                if ( pSettings->mLaunchThunderbird && !mThunderbirdProcess && mThunderbirdStartTime < QDateTime::currentDateTime() )
-                {
+                if (settings->mLaunchThunderbird && !mThunderbirdProcess &&
+                    mThunderbirdStartTime < QDateTime::currentDateTime()) {
                     startThunderbird();
 
                     // Hide after?
-                    if ( pSettings->mHideWhenStarted )
+                    if (settings->mHideWhenStarted) {
                         mThunderbirdWindowHide = true;
+                    }
                 }
-            }
-            else
-            {
+            } else {
                 // It has run before, but not running now. Should we restart?
-                if ( pSettings->mRestartThunderbird && !mThunderbirdProcess )
-                {
+                if (settings->mRestartThunderbird && !mThunderbirdProcess) {
                     startThunderbird();
 
                     // Hide after?
-                    if ( pSettings->mHideWhenRestarted )
+                    if (settings->mHideWhenRestarted) {
                         mThunderbirdWindowHide = true;
+                    }
                 }
             }
         }
@@ -314,18 +363,19 @@ void TrayIcon::blinkTimeout()
     }
     else
     {
+        Settings* settings = BirdtrayApp::get()->getSettings();
         // We are either not blinking at all, or doing no transition
-        // this depends on nonzero pSettings->mBlinkSpeed
-        if ( pSettings->mBlinkSpeed != 0 )
-        {
+        // this depends on nonzero settings->mBlinkSpeed
+        if (settings->mBlinkSpeed != 0) {
             // Flip the opacity
-            if ( mBlinkingIconOpacity == pSettings->mUnreadOpacityLevel )
-                mBlinkingIconOpacity = 1.0 - pSettings->mUnreadOpacityLevel;
-            else
-                mBlinkingIconOpacity = pSettings->mUnreadOpacityLevel;
+            if (mBlinkingIconOpacity == settings->mUnreadOpacityLevel) {
+                mBlinkingIconOpacity = 1.0 - settings->mUnreadOpacityLevel;
+            } else {
+                mBlinkingIconOpacity = settings->mUnreadOpacityLevel;
+            }
+        } else {
+            mBlinkingIconOpacity = settings->mUnreadOpacityLevel;
         }
-        else
-            mBlinkingIconOpacity = pSettings->mUnreadOpacityLevel;
     }
 
     updateIcon();
@@ -338,11 +388,20 @@ void TrayIcon::actionQuit()
 
 void TrayIcon::actionSettings()
 {
-    DialogSettings dlg;
-
-    if ( dlg.exec() == QDialog::Accepted )
-    {
-        pSettings->save();
+    if (settingsDialog != nullptr) {
+        settingsDialog->show();
+        settingsDialog->raise();
+        settingsDialog->activateWindow();
+        return;
+    }
+    settingsDialog = new DialogSettings();
+    connect(settingsDialog, &QDialog::finished, this, [=](int result) {
+        settingsDialog->deleteLater();
+        settingsDialog = nullptr;
+        if (result != QDialog::Accepted) {
+            return;
+        }
+        BirdtrayApp::get()->getSettings()->save();
 
         if ( !mUnreadMonitor )
             createUnreadCounterThread();
@@ -354,9 +413,11 @@ void TrayIcon::actionSettings()
         enableBlinking( false );
 
         updateIcon();
+        // TODO: Update on thunderbird path setting change
 
         emit settingsChanged();
-    }
+    });
+    settingsDialog->show();
 }
 
 void TrayIcon::actionActivate()
@@ -397,23 +458,29 @@ void TrayIcon::actionUnsnooze()
     updateIcon();
 }
 
-void TrayIcon::actionNewEmail()
-{
+void TrayIcon::actionNewEmail() {
+    Settings* settings = BirdtrayApp::get()->getSettings();
+
+    QString executable;
     QStringList args;
+
+    if ( !settings->getStartThunderbirdCmdline( executable, args ) )
+        return;
+
     args << "-compose";
 
-    if ( !pSettings->mNewEmailData.isEmpty() )
-    {
-        QAction * action = (QAction *) sender();
-        int index = ( action->data().toInt() );
-
-        if ( index <  0 || index > pSettings->mNewEmailData.size() - 1 )
-            return;
-
-        args << pSettings->mNewEmailData[index].asArgs();
+    if (!settings->mNewEmailData.isEmpty()) {
+        auto* action = (QAction*) sender();
+        if (action->data().isValid()) {
+            int index = (action->data().toInt());
+            if (index < 0 || index > settings->mNewEmailData.size() - 1) {
+                return;
+            }
+            args << settings->mNewEmailData[index].asArgs();
+        }
     }
 
-    QProcess::startDetached( pSettings->getThunderbirdExecutablePath(), args );
+    QProcess::startDetached(executable, args);
 }
 
 void TrayIcon::actionIgnoreEmails()
@@ -422,18 +489,18 @@ void TrayIcon::actionIgnoreEmails()
     updateIgnoredUnreads();
 }
 
-void TrayIcon::actionSystrayIconActivated(QSystemTrayIcon::ActivationReason reason)
-{
-    if ( reason == QSystemTrayIcon::Trigger )
-    {
-        if ( pSettings->mShowHideThunderbird )
+void TrayIcon::actionSystrayIconActivated(QSystemTrayIcon::ActivationReason reason) {
+    if (reason == QSystemTrayIcon::Trigger) {
+        if (BirdtrayApp::get()->getSettings()->mShowHideThunderbird) {
             actionActivate();
+        }
     }
 }
 
 void TrayIcon::createMenu()
 {
-    QMenu * menu = new QMenu();
+    Settings* settings = BirdtrayApp::get()->getSettings();
+    mSystrayMenu->clear();
 
     // Show and hide action
     mMenuShowHideThunderbird = new QAction( tr("Hide Thunderbird"), this );
@@ -442,36 +509,31 @@ void TrayIcon::createMenu()
     // We start with disabled action, and enable it once the window is detected
     mMenuShowHideThunderbird->setEnabled( false );
 
-    menu->addAction( mMenuShowHideThunderbird );
-    menu->addSeparator();
+    mSystrayMenu->addAction( mMenuShowHideThunderbird );
+    mSystrayMenu->addSeparator();
 
     // New email could be either a single action, or menu depending on settings
-    if ( pSettings->mNewEmailMenuEnabled )
-    {
-        if ( !pSettings->mNewEmailData.isEmpty() )
-        {
+    if (settings->mNewEmailMenuEnabled) {
+        if (!settings->mNewEmailData.isEmpty()) {
             // A submenu
-            QMenu * newemails = new QMenu( tr("New Email") );
-
-            for ( int index = 0; index < pSettings->mNewEmailData.size(); index++ )
-            {
-                QAction * a = new QAction( pSettings->mNewEmailData[index].menuentry(), this );
-                connect( a, &QAction::triggered, this, &TrayIcon::actionNewEmail );
-
+            auto* newEmails = new QMenu(tr("New Email"));
+            auto* action = new QAction(tr("Blank"), this);
+            connect(action, &QAction::triggered, this, &TrayIcon::actionNewEmail);
+            newEmails->addAction(action);
+            newEmails->addSeparator();
+            for (int index = 0; index < settings->mNewEmailData.size(); index++) {
+                action = new QAction(settings->mNewEmailData[index].menuentry(), this);
+                connect(action, &QAction::triggered, this, &TrayIcon::actionNewEmail);
                 // Remember the delay in the action itself
-                a->setData( index );
-                newemails->addAction( a );
+                action->setData(index);
+                newEmails->addAction(action);
             }
-
-            menu->addMenu( newemails );
-        }
-        else
-        {
+            mSystrayMenu->addMenu(newEmails);
+        } else {
             // A single action
-            menu->addAction( tr("New Email Message"), this, SLOT( actionNewEmail()) );
+            mSystrayMenu->addAction(tr("New Email Message"), this, SLOT(actionNewEmail()));
         }
-
-        menu->addSeparator();
+        mSystrayMenu->addSeparator();
     }
 
     // Snoozer times map, for easy editing. The first parameter is in seconds
@@ -497,37 +559,34 @@ void TrayIcon::createMenu()
     }
 
     // And add snoozing menu itself
-    menu->addMenu( snooze );
+    mSystrayMenu->addMenu( snooze );
 
     // Unsnooze menu item is unvisible by default
     mMenuUnsnooze = new QAction( tr("Unsnooze"), this );
     connect( mMenuUnsnooze, &QAction::triggered, this, &TrayIcon::actionUnsnooze );
 
-    menu->addAction( mMenuUnsnooze );
+    mSystrayMenu->addAction( mMenuUnsnooze );
     mMenuUnsnooze->setVisible( false );
 
     // Add the ignore action
-    if ( pSettings->mAllowSuppressingUnreads )
-    {
+    if (settings->mAllowSuppressingUnreads) {
         mMenuIgnoreUnreads = new QAction( tr("Ignore unread emails"), this );
         connect( mMenuIgnoreUnreads, &QAction::triggered, this, &TrayIcon::actionIgnoreEmails );
 
-        menu->addAction( mMenuIgnoreUnreads );
+        mSystrayMenu->addAction( mMenuIgnoreUnreads );
     }
     else
         mMenuIgnoreUnreads = 0;
 
-    menu->addSeparator();
+    mSystrayMenu->addSeparator();
 
     // Some generic actions
-    menu->addAction( tr("Settings..."), this, SLOT(actionSettings()) );
+    mSystrayMenu->addAction( tr("Settings..."), this, SLOT(actionSettings()) );
 
-    menu->addSeparator();
+    mSystrayMenu->addSeparator();
 
     // And exit
-    menu->addAction( tr("Quit"), this, SLOT(actionQuit()) );
-
-    setContextMenu( menu );
+    mSystrayMenu->addAction( tr("Quit"), this, SLOT(actionQuit()) );
 }
 
 void TrayIcon::createUnreadCounterThread()
@@ -542,8 +601,16 @@ void TrayIcon::createUnreadCounterThread()
 
 void TrayIcon::startThunderbird()
 {
-    QString thunderbirdExePath = pSettings->getThunderbirdExecutablePath();
-    Utils::debug("Starting Thunderbird as '%s'", qPrintable( thunderbirdExePath ) );
+    QString executable;
+    QStringList args;
+
+    if ( !BirdtrayApp::get()->getSettings()->getStartThunderbirdCmdline( executable, args ) )
+    {
+        Utils::debug("Failed to get Thunderbird command-line" );
+        return;
+    }
+
+    Utils::debug("Starting Thunderbird as '%s %s'", qPrintable(executable), qPrintable(args.join(' ')));
 
     if ( mThunderbirdProcess )
         mThunderbirdProcess->deleteLater();
@@ -555,16 +622,24 @@ void TrayIcon::startThunderbird()
     connect( mThunderbirdProcess, &QProcess::errorOccurred, this, &TrayIcon::tbProcessError );
 #endif
 
-    mThunderbirdProcess->start( thunderbirdExePath );
+    mThunderbirdProcess->start(executable, args);
 }
 
 void TrayIcon::tbProcessError(QProcess::ProcessError )
 {
-    QMessageBox::critical( 0,
-                           tr("Cannot start Thunderbird"),
-                           tr("Error starting Thunderbird as %1:\n\n%2")
-                                .arg( pSettings->getThunderbirdExecutablePath() )
-                                .arg( mThunderbirdProcess->errorString() ) );
+#ifdef Q_OS_WIN
+    if (mThunderbirdUpdaterProcess->attach() == AttachResult::SUCCESS)
+    {
+        return;
+    }
+#endif /* Q_OS_WIN */
+
+    QMessageBox::critical(nullptr,
+            tr("Cannot start Thunderbird"),
+            tr("Error starting Thunderbird as '%1 %2':\n\n%3")
+                    .arg( mThunderbirdProcess->program() )
+                    .arg( mThunderbirdProcess->arguments().join(' ') )
+                    .arg( mThunderbirdProcess->errorString()) );
 
     // We keep the mThunderbirdProcess pointer, so the process is not restarted again
 }
@@ -575,18 +650,63 @@ void TrayIcon::tbProcessFinished(int, QProcess::ExitStatus)
     // in which case it is restarted in updateState(), or that we started TB
     // and the active instance was activated (and our instance exited).
     // Thus we just destroy the process later, to let updateState() make decision
+#ifdef Q_OS_WIN
+    if (mThunderbirdUpdaterProcess->attach() == AttachResult::SUCCESS) {
+        return;
+    }
+#endif /* Q_OS_WIN */
     mThunderbirdProcess->deleteLater();
-    mThunderbirdProcess = 0;
+    mThunderbirdProcess = nullptr;
 }
 
+#ifdef Q_OS_WIN
+void TrayIcon::tbUpdaterProcessFinished(const ProcessHandle::ExitReason& exitReason)
+{
+    if (exitReason.isError()) {
+        QMessageBox::critical(
+                nullptr, tr("Cannot start Thunderbird"),
+                tr("Error starting Thunderbird, because we could not attach to the updater:\n\n%1")
+                .arg( exitReason.getErrorDescription() ) );
+        return;
+    }
+    // The updater will start Thunderbird automatically
+    mThunderbirdProcess->deleteLater();
+    mThunderbirdProcess = nullptr;
+}
+#endif /* Q_OS_WIN */
+
 void TrayIcon::onQuit() {
-    if ( mWinTools && mWinTools->isHidden() )
+    if (mWinTools && mWinTools->isHidden()) {
         mWinTools->show();
-    
-    if ( pSettings->mExitThunderbirdWhenQuit )
-    {
-        if ( mWinTools )
+    }
+    if (BirdtrayApp::get()->getSettings()->mExitThunderbirdWhenQuit) {
+        if (mWinTools) {
             mWinTools->closeWindow();
+        }
+    }
+}
+
+void TrayIcon::onAutoUpdateCheckFinished(const QString &errorMessage) {
+    AutoUpdater* autoUpdater = BirdtrayApp::get()->getAutoUpdater();
+    if (errorMessage.isNull()) {
+        disconnect(autoUpdater, &AutoUpdater::onCheckUpdateFinished,
+                   this, &TrayIcon::onAutoUpdateCheckFinished);
+    } else if (networkConnectivityManager == nullptr) {
+        networkConnectivityManager = new QNetworkConfigurationManager();
+        networkConnectivityManager->updateConfigurations();
+        auto callback = [=](const QNetworkConfiguration &config) {
+            if (config.state() == QNetworkConfiguration::Active) {
+                if (networkConnectivityManager != nullptr) {
+                    networkConnectivityManager->deleteLater();
+                    networkConnectivityManager = nullptr;
+                }
+                autoUpdater->checkForUpdates();
+            }
+        };
+        connect(networkConnectivityManager, &QNetworkConfigurationManager::configurationChanged,
+                this, callback);
+        connect(networkConnectivityManager, &QNetworkConfigurationManager::configurationAdded,
+                this, callback);
     }
 }
 
@@ -612,3 +732,10 @@ void TrayIcon::updateIgnoredUnreads()
             mMenuIgnoreUnreads->setText( tr("Ignore unread emails") );
     }
 }
+
+void TrayIcon::doAutoUpdateCheck() {
+    AutoUpdater* autoUpdater = BirdtrayApp::get()->getAutoUpdater();
+    connect(autoUpdater, &AutoUpdater::onCheckUpdateFinished,
+            this, &TrayIcon::onAutoUpdateCheckFinished);
+    autoUpdater->checkForUpdates();
+}
